function data = feedback(allpars,xrand,yrand,numsamples,numexpts)
%FEEDBACK.M
% Simulate a model for positive transcriptional feedback. 
%
%   dx/dt = lambda*y^m/(theta^m+y^m)-mu1*x
%   dy/dt = delta*x-mu2*y
%
% We choose a parameter set and run the simulation. 
% We then use DEVAL to extract the state at a subset of times (as if we had
% experimental samples), at which we add noise to mimic experimental error
% and intrinisc fluctuations. 
%
% The relative magnitude of random perturbations to the true solution can
% be set when calling the function, or left as default (zeros). 
%
% allpars is a vector or parameters [lambda,m,theta,mu1,delta,mu2]
% if it is not passed, the default set is used

% if not passed, set the default parameters, relative magnitude of random 
% perturbations and number of samples
if nargin < 5
    % set the default model parameters
    lambda = 0.5;
    m = 3;
    theta = 0.005;
    mu1 = 0.1;
    delta = 0.2;
    mu2 = 0.5;
    % the initial conditions are also paramaters
    x0 = 0.05;
    y0 = 0.01;
    xrand = 0;
    yrand = 0;
    numexpts = 1;
    numsamples = 4;
else
    lambda=allpars(1);
    m=allpars(2);
    theta=exp(allpars(3));
    mu1=allpars(4);
    delta=allpars(5);
    mu2=allpars(6);
    x0=allpars(7);
    y0=allpars(8);
end

disp(['Running FEEDBACK.M ... xrand = ' num2str(xrand) ', yrand = ' num2str(yrand)])

% initialise the random number generator so our "noise" is reproducible
rand('twister',1)

% set the time interval
tmax = 20;
tspan = [0 tmax];
% a finely resolved time range for plotting the full solution
t = linspace(0,tmax,100);
% coarsely spaced "experimental" sample times
tsample = linspace(0,tmax,numsamples);

% run the simulation, generating a solution structure this time so we can
% use DEVAL to extract the sample timepoints
sol = ode45(@f,tspan,[x0 y0]);
x = deval(sol,t,1);
y = deval(sol,t,2);

% plot the simulation and the sample data
figure(1)
clf
set(gca,'fontsize',14)
hold on
plot(t,x,t,y,'-','linewidth',2)
grid on
% add labels
xlabel('time','fontsize',16)
ylabel('x(t), y(t)','fontsize',16)
title('Positive feedback model - generate data','fontsize',16)
axis tight

% open a file in text write mode
fid = fopen('feedback.txt','wt');
% generate numexpts sets of sampled data and write to a file
for i=1:numexpts
    % set the relative noise in x and y (random numbers in [-xrand,xrand], etc)
    xnoise = -xrand + 2*xrand*rand(size(tsample));
    ynoise = -yrand + 2*yrand*rand(size(tsample));
    % extract sample and add noise
    xsample = deval(sol,tsample,1).*(1+xnoise);
    ysample = deval(sol,tsample,2).*(1+ynoise);
    % plot the sample data
    plot(tsample,xsample,'*',tsample,ysample,'*','linewidth',2,'markersize',10)
    % return the sample data
    % use transpose to get time down the rows
    data = [tsample; xsample; ysample]';
    fprintf(fid,'%6.4f\t %6.4f\t %6.4f\n',data');
    fprintf(fid,'\n');
end
% close the file
fclose(fid);

% add a key? 
legend('x^{true}(t)','y^{true}(t)','x^{data}(t)','y^{data}(t)','location','best')

    % ---------------------------------------------------------------
    function dxdt=f(t,x)
    dxdt = [lambda*x(2)^m/(x(2)^m+theta^m)-mu1*x(1);
        delta*x(1)-mu2*x(2)];
    end

end

