function feedbackfit
%FEEDBACKFIT.M
% 
% function applying FMINSEARCH to fit a model for positive transcriptional
% feedback to simulated data. FMINSEARCH uses the Nelder-Mead simplex method. 
%
% We try to fit parameters for the model
%   dx/dt = lambda*y^m/(theta^m+y^m)-mu1*x
%   dy/dt = delta*x-mu2*y
%
% The data is generated by "feedback.m"

disp(sprintf('\n********************\nRunning FEEDBACKFIT.M ...\n********************'))

% ---------------------------------------------------------------
% ***************************************************************
% CHANGE THESE: TRUE SYSTEM/MODEL PARAMETERS
% See how the changing the parameters affects the fitting process
% ---------------------------------------------------------------
% set the model parameters to generate "experimental" data
% the set of parameter values, [lambda,m,theta,mu1,delta,mu2,x0,y0] 
lambda = 0.5;
m = 3;
theta = 0.005;
mu1 = 0.1;
delta = 0.2;
mu2 = 0.5;
% the initial conditions are also paramaters
x0 = 0.05;
y0 = 0.01;
% noise in x
xrand = 0.05;
% noise in y
yrand = 0.05;
% number of time points
numsamples = 6;
% which parameters to fit - the rest are assumed known
ifitpars = [1 2 3 4 5 6 7 8];
% ifitpars = [1   3   5   7  ];
% this search needs a reasonable initial guess - here are a couple
allguess1 = [0.5,2,log(0.005),0.5,2,0.1,0.005,0.01];
allguess2 = [0.5,3,log(0.005),0.5,2,0.1,0.005,0.01];
allguess3 = [0.5,3,log(0.15),0.5,2,0.1,0.005,0.01];
% try the routine with one of the guesses (select those that are not fixed)
fitpars = allguess1(ifitpars);
% ***************************************************************
% ---------------------------------------------------------------

% allpars is the set of parameter values, [lambda,m,theta,mu1,delta,mu2,x0,y0] 
allpars = [lambda,m,log(theta),mu1,delta,mu2,x0,y0];
% store the original parameters for comparison
storepars = [lambda,m,theta,mu1,delta,mu2,x0,y0];

% generate data in FEEDBACK.M
data = feedback(allpars,xrand,yrand,numsamples,1);

% assign the time data to t
t=data(:,1);
% assign the x and y "experimental" data
x_exp=data(:,2);
y_exp=data(:,3);

% look for the best fit to the numerical solution to the ODE
options = optimset('maxfunevals',20000,'maxiter',2000,'TolFun',1e-10);
[fitpars,fval] = fminsearch(@(x) feedbackError(x,t,x_exp,y_exp),fitpars,options);
% define yy to be the solution at the time points in "t"
allpars(ifitpars) = fitpars;
lambda=allpars(1);
m=allpars(2);
theta=exp(allpars(3));
mu1=allpars(4);
delta=allpars(5);
mu2=allpars(6);
x0=allpars(7);
y0=allpars(8);
% generate a simulation with these parameters for plotting
[t,y] = ode45(@f,[0 max(t)],[x0 y0]);
% plot the solution on the same axis to see how well it fits the data
plot(t,y,'--','linewidth',2)

% add labels
xlabel('time','fontsize',16)
ylabel('x(t), y(t)','fontsize',16)
title('Positive feedback model - fit to data','fontsize',16)
axis tight
% add a key? 
legend('x^{true}(t)','y^{true}(t)','x^{data}(t)','y^{data}(t)','x^{fit}(t)','y^{fit}(t)','location','best')

% strings for par names
parnames = {'lambda' 'm' 'theta' 'mu1' 'delta' 'mu2' 'x0' 'y0'};
allpars(3) = theta;
disp(' ')
for i=1:length(parnames)        
    disp(sprintf('%s \t = %g, %g',parnames{i},storepars(i),allpars(i)))
end

disp(sprintf('\nFinished FEEDBACKFIT.M ...\n'))

    % ---------------------------------------------------------------
    % function defining the feedback model rates of change
    % ---------------------------------------------------------------
    function dxdt=f(t,x)
        dxdt = [lambda*x(2)^m/(x(2)^m+theta^m)-mu1*x(1);
            delta*x(1)-mu2*x(2)];
    end

    % ---------------------------------------------------------------
    % function defining the error between the feedback data 
    % and model solution
    % ---------------------------------------------------------------
    function lse=feedbackError(fitpars,t,x_exp,y_exp)
        %update the values of the fitted parameters
        allpars(ifitpars) = fitpars;
        %extract the elements of allpars 
        lambda=allpars(1);
        m=allpars(2);
        theta=exp(allpars(3));
        mu1=allpars(4);
        delta=allpars(5);
        mu2=allpars(6);
        x0=allpars(7);
        y0=allpars(8);
        %use to create a new curve
        [t,y] = ode45(@f,t,[x0 y0]);
        %calculate objective function (squared error)
        lse=sum((y(:,1)-x_exp).^2)+sum((y(:,2)-y_exp).^2);
    end

end
