function yeastfit
%YEASTFIT.M
% 
% function applying FMINSEARCH to fit the generalised Logistic model to
% Yeast growth data. FMINSEARCH uses the Nelder-Mead simplex method. 
%
% We try to fit parameters for a generalised logistic growth model
% dx/dt = r*x*(1-(x/K)^v)
% which has solution
% x(t) = K/(1+((K/x0)^v-1)*exp(-r*v*t))^(1/v)
% We try fitting the analytical and simulated solutions to the average of
% three experiments

disp(sprintf('\n********************\nRunning YEASTFIT.M ...\n********************'))

% ---------------------------------------------------------------
% READ IN AND PROCESS THE DATA
% ---------------------------------------------------------------
% turn off MATLAB warnings
warning off
% read in the data from Sheet2 of the Excel spreadsheet
[NUMERIC,TXT,RAW]=xlsread('CEN-PK113-5D.xls','Sheet2','','basic');
% the first row of NUMERIC includes a header row, so discard this
yeast = NUMERIC(2:end,:);
% yeast should now have 94 rows and 6 columns (time, 3 replicates, mean and stddev)
% what is the index for time=1500? 
i1500 = find(yeast(:,1)==1500); 
% beyond this point the data looks dodgy...
yeast = yeast(1:i1500,:);
% ---------------------------------------------------------------

% ---------------------------------------------------------------
% ***************************************************************
% CHANGE THESE: INITIAL GUESSES AT MODEL PARAMETERS
% See how the parameters found and their sensitivity changes
% ---------------------------------------------------------------
% pars is the set of parameter values, [r,K,v,x0] 
% NOTE: the initial value is a parameter to fit
% for initial guess we could use the mean from the data
x0 = mean(yeast(1,2:4));
% this search needs a reasonable initial guess - here are a couple
guess1 = [0.1,1,0.1,x0];
guess2 = [0.1,1,1,0.1];
guess3 = [1,1,0.01,0.1];
% try the routines with one of the initial guesses
pars = guess1;
% ***************************************************************
% ---------------------------------------------------------------

% ---------------------------------------------------------------
% LEAVE THE REST ALONE...
% ---------------------------------------------------------------
% assign the individual parameter values
r=pars(1);
K=pars(2);
v=pars(3);
x0=pars(4);
% print the initial values to the screen
disp(sprintf('Initial guess: r=%g, K=%g, v=%g, x0=%g.\n',r,K,v,x0))
% open figure 1
figure(1)
% clear the figure
clf
set(gca,'fontsize',12)
% plot the three replicates: optical density as a function of time 
plot(yeast(:,1),yeast(:,2:4),'linewidth',2)
% freeze the plot so we can add additional plots
hold on
% put a grid on the plot
grid on

% assign the time data to t
t=yeast(:,1);
% the fifth column is the mean o.d.
x_exp=yeast(:,5);

% ---------------------------------------------------------------
% look for the best fit for the analytical solution to the ODE 
% ---------------------------------------------------------------
[pars,fval] = fminsearch(@(x) yeastError(x,t,x_exp),pars);
% define xx to be the solution at the time points in "t"
disp(sprintf('Generalised logistic model fit: r=%g, K=%g, v=%g, x0=%g. Error=%g',r,K,v,x0,fval))
xx = genlogsol(t);
% plot the solution on the same axis to see how well it fits the data
plot(t,xx,'k','linewidth',2)
% generate a string for the key
funstring = sprintf('Gen. Logistic fit: r=%0.2g, K=%0.2g, v=%0.2g, x0=%0.2g',r,K,v,x0);
% check the model sensitivity for these parameters
yeastErrorsens

% add labels
xlabel('time (min)','fontsize',16)
ylabel('Optical density','fontsize',16)
title('Yeast growth (Saccharomyces cerevisiae CEN.PK113)','fontsize',16)
axis([0 max(t) 0 1.05])
% add a key? 
legend('Expt 1','Expt 2','Expt 3',funstring,'location','southoutside')

    % ---------------------------------------------------------------
    % function defining the solution to the generalised logistic model
    % ---------------------------------------------------------------
    function x=genlogsol(t)
        x = K./(1+((K/x0)^v-1)*exp(-r*v*t)).^(1/v);
    end

    % ---------------------------------------------------------------
    % function defining the error between the yeast growth data 
    % and model solution
    % ---------------------------------------------------------------
    function lse=yeastError(pars,t,x_exp)
        %extract the elements of the pars 
        r=pars(1);
        K=pars(2);
        v=pars(3);
        x0=pars(4);
        %use to create a new curve
        x = genlogsol(t);
        %calculate objective function (squared error)
        lse=sum((x-x_exp).^2);
    end

    % ---------------------------------------------------------------
    % function defining the sensitivity of the error to fractional
    % increases and decreases in model parameters
    % ---------------------------------------------------------------
    function y=yeastErrorsens
        % try changing each parameter in turn by +/-fraction
        % need to store the old parameter values as yeastError, etc, change them
        oldpars = pars;
        fractions = [0.01 -0.01 0.1 -0.1];
        % store the sensitivities, the first row is the fractional changes
        storesens = [0 fractions];
        for i=1:length(pars)
            isens = i;
            for ff = fractions
                pars = oldpars;
                pars(i) = (1+ff)*oldpars(i);
                newfval = yeastError(pars,t,x_exp);
                % sensitivity = (dphi/phi)/(dp/p) = dphi/(phi*ff) where phi is the
                % "feature" and p is the parameter
                sens = (newfval-fval)/(fval*ff);
                % accumulate sensitivities for parameter i
                isens = [isens, sens];
            end
            % accumulate sensitivities for all parameters
            storesens = [storesens; isens];
        end
        disp('Matrix of sensitivities: top row=fractional parameter change, 1st column=parameter number');
        disp(storesens);
        pars = oldpars;
    end

end
