function yeastfitlogistic
%YEASTFITLOGISITC.M
% 
% function applying FMINSEARCH to fit the standard Logistic model to
% Yeast growth data. FMINSEARCH used the Nelder-Mead simplex method. 
%
% We try to fit parameters for a logistic growth model
% dx/dt = r*x*(1-x/K)
% which has solution
% x(t) = K/(1+(K/x0-1)*exp(-r*t))
%
% We fix x0 (the initial population size), and the parameter vector is [r,K]

disp(sprintf('\n********************\nRunning YEASTFITLOGISTIC.M ...\n********************'))

% ---------------------------------------------------------------
% READ IN AND PROCESS THE DATA
% ---------------------------------------------------------------
% turn off MATLAB warnings
warning off
% read in the data from Sheet2 of the Excel spreadsheet
[NUMERIC,TXT,RAW]=xlsread('CEN-PK113-5D.xls','Sheet2','','basic');
% the first row of NUMERIC includes a header row, so discard this
yeast = NUMERIC(2:end,:);
% yeast should now have 94 rows and 6 columns (time, 3 replicates, mean and stddev)
% what is the index for time=1500? 
i1500 = find(yeast(:,1)==1500); 
% beyond this point the data looks dodgy...
yeast = yeast(1:i1500,:);
% ---------------------------------------------------------------

% ---------------------------------------------------------------
% ***************************************************************
% CHANGE THESE: INITIAL GUESSES AT MODEL PARAMETERS
% See how the parameters found and their sensitivity changes
% ---------------------------------------------------------------
% pars is the set of parameter values, [r,K,v,x0] 
% this search needs a reasonable initial guess - here are a couple
guess1 = [0.01,0.7];
guess2 = [0.01,0.8];
guess3 = [0.14,0.9];
% try the routines with one of the guesses
pars = guess1;
% ---------------------------------------------------------------

% ---------------------------------------------------------------
% LEAVE THE REST ALONE...
% ---------------------------------------------------------------
% assign the individual parameter values
r=pars(1);
K=pars(2);
% print the initial values to the screen
disp(sprintf('Initial guess: r=%g, K=%g.\n',r,K))
% open figure 1
figure(2)
% clear the figure
clf
set(gca,'fontsize',12)
% plot the three replicates: optical density as a function of time 
plot(yeast(:,1),yeast(:,2:4),'linewidth',2)
% freeze the plot so we can add additional plots
hold on
% put a grid on the plot
grid on

% assign the time data to t
t=yeast(:,1);
% the fifth column is the mean o.d.
x_exp=yeast(:,5);
% NOTE: we fix the initial value using the mean from the data
x0 = mean(yeast(1,2:4));

% ---------------------------------------------------------------
% look for the best fit for the analytical solution to the ODE 
% ---------------------------------------------------------------
options = optimset('maxfunevals',2000,'maxiter',2000);
% look for the best fit to the analytical solution to the ODE 
[pars,fval] = fminsearch(@(x) yeastError(x,t,x_exp),pars,options);
% define yy to be the solution at the time points in "t"
r=pars(1);
K=pars(2);
disp(sprintf('Logistic model fit to yeast data:  r=%g, K=%g, x0=%g. Error=%g',r,K,x0,fval))
yy = logsol(t);
% plot the solution on the same axis to see how well it fits the data
plot(t,yy,'k','linewidth',2)
% generate a string for the key
fitstring = sprintf('Logistic model fit: r=%0.3g, K=%0.3g',r,K);
% check the model sensitivity for these parameters
yeastErrorsens
% add labels
xlabel('time (min)','fontsize',16)
ylabel('Optical density','fontsize',16)
title('Yeast growth (Saccharomyces cerevisiae CEN.PK113)','fontsize',16)
axis([0 max(t) 0 1.05])
% add a key? 
legend('Expt 1','Expt 2','Expt 3',fitstring,'location','southoutside')

    % ---------------------------------------------------------------
    % function defining the solution to the logistic model
    % ---------------------------------------------------------------
    function x=logsol(t)
        x = K./(1+(K/x0-1)*exp(-r*t));
    end

    % ---------------------------------------------------------------
    % function defining the error between the yeast growth data 
    % and model solution
    % ---------------------------------------------------------------
    function lse=yeastError(pars,t,x_exp)
        %extract the elements of the parameter vector pars 
        r=pars(1);
        K=pars(2);
        %use to create a new curve
        x = logsol(t);
        %calculate objective function (squared error)
        lse=sum((x-x_exp).^2);
    end

    % ---------------------------------------------------------------
    % function defining the sensitivity of the error to fractional
    % increases and decreases in model parameters
    % ---------------------------------------------------------------
    function y=yeastErrorsens
        % try changing each parameter in turn by +/-fraction
        % need to store the old parameter values as the yeastError function
        % changes them
        oldpars = pars;
        % sets of fractional parameter changes
        fractions = [0.01 -0.01 0.1 -0.1];
        % store the sensitivities, the first row is the fractional changes
        storesens = [0 fractions];
        for i=1:length(pars)
            isens = i;
            for ff = fractions
                pars = oldpars;
                pars(i) = (1+ff)*oldpars(i);
                newfval = yeastError(pars,t,x_exp);
                % sensitivity = (dphi/phi)/(dp/p) = dphi/(phi*ff) where phi is the
                % "feature" and p is the parameter
                sens = (newfval-fval)/(fval*ff);
                % accumulate sensitivities for parameter i
                isens = [isens, sens];
            end
            % accumulate sensitivities for all parameters
            storesens = [storesens; isens];
        end
        disp('Matrix of sensitivities: top row=fractional parameter change, 1st column=parameter number');
        disp(storesens);
        pars = oldpars;
    end

end
