function yeastscanlogistic
%YEASTSCANLOGISTIC.M
% 
% function mapping out the error for a model of yeast growth data 
% as the two parameters vary in the standard logistic growth model  
%
% We try to fit parameters for the standard logistic growth model
% dx/dt = r*x*(1-x/K)
% which has solution
% x(t) = K/(1+(K/x0-1)*exp(-r*t))
%
% Here we fix x0 according to the first time point and 
% we find the error as r and K vary 
% There is a clear global minimum. 

% ---------------------------------------------------------------
% read in and process the data
% ---------------------------------------------------------------
% read in the data from Sheet2 of the Excel spreadsheet
[NUMERIC,TXT,RAW]=xlsread('CEN-PK113-5D.xls','Sheet2','','basic');
% the first row of NUMERIC includes a header row, so discard this
yeast = NUMERIC(2:end,:);
% yeast should now have 94 rows and 6 columns (time, 3 replicates, mean and stddev)
% what is the index for time=1500? 
i1500 = find(yeast(:,1)==1500); 
% beyond this point the data looks dodgy...
yeast = yeast(1:i1500,:);

% ---------------------------------------------------------------
% scan the (r,K) parameter space calculating the error
% ---------------------------------------------------------------
% assign the time data to t
t=yeast(:,1);
% the fifth column is the mean optical density
x_exp=yeast(:,5);
% for initial population we use the mean from the 1st data point
x0 = mean(yeast(1,2:4));
% for yeastscanlogistic.eps
% set ranges for r and K
rr = 0.001:0.001:0.1;
KK = 0.01:0.01:1;
% loop through all pairs of r and v values, calculating the squared error
% for each
for i = 1:length(rr)
    for j = 1:length(KK)
        r = rr(i);
        K = KK(j);
        lsee(i,j) = yeastError(t,x_exp,r,K,x0);
    end
end
% find the minimum error
minError = min(min(lsee));
% find the parameter pair that gives that minimum error
[i,j] = find(lsee==minError);
% generate a string to display the parameters that give the minimum error
funstring = sprintf('Logistic minimum error = %5.3g, for (r,K)=(%0.2g,%0.2g)',lsee(i,j),rr(i),KK(j));

% ---------------------------------------------------------------
% plot the (r,K) parameter space showing the error
% ---------------------------------------------------------------
figure(1)
clf
% plot an image with colour indicating the size of the error 
% IMAGESC colours each pixel according to the data
imagesc(KK,rr,lsee)
% add a bar indicating the relationship between colour and the error
colorbar
hold on
% plot the point corresponding to the minimum error
plot(KK(j),rr(i),'*w','linewidth',2,'markersize',10)
% add contours as well at errors of 0.5, 2 and 5
[cs,h] = contour(KK,rr,lsee,[0.5 2 5],'w'); 
% label the contours
clabel(cs,h,'fontsize',15,'color','w','rotation',0,'labelspacing',288)
% label the axes
xlabel('K','fontsize',16)
ylabel('r','fontsize',16)
set(gca,'fontsize',16)
set(gca,'ydir','normal')
title('Squared error for yeast data, logistic growth model')

% ---------------------------------------------------------------
% plot the data and the best fit
% ---------------------------------------------------------------
% open figure 1
figure(2)
clf
set(gca,'fontsize',12)
% plot the three replicates: optical density as a function of time 
plot(yeast(:,1),yeast(:,2:4),'linewidth',2)
% freeze the plot so we can add additional plots
hold on
% put a grid on the plot
grid on
% generate the corresponding solution for the best fit parameters
xx = logsol(t,rr(i),KK(j),x0);
% plot the solution on the same axis to see how well it fits the data
plot(t,xx,'k','linewidth',2)
% add labels
xlabel('time (min)','fontsize',16)
ylabel('Optical density','fontsize',16)
title('Yeast growth (Saccharomyces cerevisiae CEN.PK113)','fontsize',16)
axis([0 max(t) 0 1.05])
% add a key? 
legend('Expt 1','Expt 2','Expt 3',funstring,'location','southoutside')

    % ---------------------------------------------------------------
    % function defining the solution to the logistic model
    % ---------------------------------------------------------------
    function x=logsol(t,r,K,x0)
        x = K./(1+(K/x0-1)*exp(-r*t));
    end

    % ---------------------------------------------------------------
    % function defining the error between the yeast growth data 
    % and model solution
    % ---------------------------------------------------------------
    function lse=yeastError(t,x_exp,r,K,x0)
        %use to create a new curve
        x = logsol(t,r,K,x0);
        %calculate objective function (squared error)
        lse=sum((x-x_exp).^2);
    end

end
